@@ -1,7 +1,7 @@
 #include "loader_common.h"
 #include <ctype.h>
 
-int
+static int
 do_progress(ImlibImage * im, ImlibProgressFunction progress,
             char progress_granularity, char *pper, int *py, int y)
 {
@@ -229,7 +229,7 @@ load(ImlibImage * im, ImlibProgressFunction progress,
                          }
                     }
                   iptr = idata;
-                  if (v == 255)
+                  if (v == 0 || v == 255)
                     {
                        for (x = 0; x < w; x++)
                          {
@@ -303,7 +303,7 @@ load(ImlibImage * im, ImlibProgressFunction progress,
                          }
                     }
                   iptr = idata;
-                  if (v == 255)
+                  if (v == 0 || v == 255)
                     {
                        for (x = 0; x < w; x++)
                          {
@@ -373,10 +373,10 @@ load(ImlibImage * im, ImlibProgressFunction progress,
              for (y = 0; y < h; y++)
                {
                   if (!fread(data, w * 1, 1, f))
-                     break;
+                     goto quit_error;
 
                   ptr = data;
-                  if (v == 255)
+                  if (v == 0 || v == 255)
                     {
                        for (x = 0; x < w; x++)
                          {
@@ -415,10 +415,10 @@ load(ImlibImage * im, ImlibProgressFunction progress,
              for (y = 0; y < h; y++)
                {
                   if (!fread(data, w * 3, 1, f))
-                     break;
+                     goto quit_error;
 
                   ptr = data;
-                  if (v == 255)
+                  if (v == 0 || v == 255)
                     {
                        for (x = 0; x < w; x++)
                          {
@@ -457,7 +457,7 @@ load(ImlibImage * im, ImlibProgressFunction progress,
              for (y = 0; y < h; y++)
                {
                   if (!fread(data, w * 1, 1, f))
-                     break;
+                     goto quit_error;
 
                   ptr = data;
                   for (x = 0; x < w; x++)
@@ -490,10 +490,10 @@ load(ImlibImage * im, ImlibProgressFunction progress,
              for (y = 0; y < h; y++)
                {
                   if (!fread(data, w * 4, 1, f))
-                     break;
+                     goto quit_error;
 
                   ptr = data;
-                  if (v == 255)
+                  if (v == 0 || v == 255)
                     {
                        for (x = 0; x < w; x++)
                          {
@@ -632,15 +632,13 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
 void
 formats(ImlibLoader * l)
 {
-   char               *list_formats[] = { "pnm", "ppm", "pgm", "pbm", "pam" };
+   static const char  *const list_formats[] =
+      { "pnm", "ppm", "pgm", "pbm", "pam" };
+   int                 i;
 
-   {
-      int                 i;
+   l->num_formats = sizeof(list_formats) / sizeof(char *);
+   l->formats = malloc(sizeof(char *) * l->num_formats);
 
-      l->num_formats = (sizeof(list_formats) / sizeof(char *));
-      l->formats = malloc(sizeof(char *) * l->num_formats);
-
-      for (i = 0; i < l->num_formats; i++)
-         l->formats[i] = strdup(list_formats[i]);
-   }
+   for (i = 0; i < l->num_formats; i++)
+      l->formats[i] = strdup(list_formats[i]);
 }
